#include "../../Source/MFC_Core/stdafx.h"
#include "Shader_Fog.h"

Shader_Fog::Shader_Fog()
{
	FogEffect   = NULL;
	FogTechHandle = NULL;

	// Init shaders
	m_pVertexShader = NULL;
	m_pPixelShader	= NULL;

	m_pConstantTableVS   = NULL;
	m_pConstantTablePS  = NULL;
}

Shader_Fog::~Shader_Fog()
{

	FogEffect->Release();
	FogTechHandle = NULL;

}

bool Shader_Fog::Init(LPDIRECT3DDEVICE9 pDevice)
{
	m_pD3DDevice = pDevice;

	// Not using FFP fog
	m_pD3DDevice->SetRenderState(D3DRS_FOGVERTEXMODE, D3DFOG_NONE);
	m_pD3DDevice->SetRenderState(D3DRS_FOGTABLEMODE,  D3DFOG_NONE);
	m_pD3DDevice->SetRenderState(D3DRS_FOGENABLE,FALSE);


	HRESULT hr = 0;
	//
	// Create effect.
	//
	ID3DXBuffer* errorBuffer = 0;
	hr = D3DXCreateEffectFromFile(
		m_pD3DDevice,
		L"Effect/Shader/Fog.fx",
		0,                // no preprocessor definitions
		0,                // no ID3DXInclude interface
		D3DXSHADER_DEBUG, // compile flags
		0,                // don't share parameters
		&FogEffect,
		&errorBuffer);

	// output any error messages
	if( errorBuffer )
	{
		::MessageBox(0, (wchar_t*)errorBuffer->GetBufferPointer(), 0, 0);
		errorBuffer->Release();

		return false;
	}

	// 
	// Save Frequently Accessed Parameter Handles
	//

	FogTechHandle = FogEffect->GetTechniqueByName("Fog");

	if(FAILED(Create_LayerFogShaders()))
		return false;

	return true;
}

HRESULT Shader_Fog::Create_LayerFogShaders()
{
	//
    // If your program uses explicit binding semantics (like this one), 
    // you can create a vertex declaration using those semantics.
    //

    D3DVERTEXELEMENT9 declaration[] =
    {
        { 0, 0,  D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
        { 0, 12, D3DDECLTYPE_D3DCOLOR, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    0 },
        { 0, 16, D3DDECLTYPE_FLOAT2,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
        D3DDECL_END()
    };

    m_pD3DDevice->CreateVertexDeclaration( declaration, &m_pVertexDeclaration );

	HRESULT				hr;
	LPD3DXBUFFER        pCode;
	LPD3DXBUFFER        pErrorMsgs;

	// Compile the vertex shader
	hr = D3DXCompileShaderFromFile(L"Effect/Shader/LayeredFog.vsh", NULL, NULL, "main", "vs_1_1", 0, &pCode, &pErrorMsgs , NULL);
	{
		hr = m_pD3DDevice->CreateVertexShader( (DWORD*)pCode->GetBufferPointer(),  &m_pVertexShader);
		if(FAILED(hr))
		{
			m_pVertexShader = NULL;
			return S_FALSE;
		}
		d3d::Release(pCode);
		d3d::Release(pErrorMsgs);
	}

/*
	hr = D3DXCompileShaderFromFile(L"Effect/Shader/LayeredFog.psh", NULL, NULL, "main", "ps_1_1", 0, &pCode, &pErrorMsgs , NULL);
	{
		hr = m_pD3DDevice->CreatePixelShader( (DWORD*)pCode->GetBufferPointer(),  &m_pPixelShader );
		
	//	if(FAILED(hr))
	//	{
	//		m_pPixelShader = NULL;
	//		return S_FALSE;
	//	}
		SAFE_RELEASE(pCode);
		SAFE_RELEASE(pErrorMsgs);
	}
*/
	return S_OK;
}


//-----------------------------------------------------------------------------
// Name: SetShaderConstants()
// Desc: Is called every frame, to provide the shaders with the needed 
//       constants
//-----------------------------------------------------------------------------
HRESULT Shader_Fog::SetShaderConstants(D3DXVECTOR3 eyePosition, int Color)
{
	D3DXMATRIX				m_matWorldViewProj;				// WorldMatrix
	D3DXMATRIX				m_matWorldView;				// ViewMatrix
	D3DXMATRIX				m_matProj;				// ProjectionMatrix
	D3DXMATRIX				m_matWorld;				// WorldMatrix

	m_pD3DDevice->GetTransform(D3DTS_WORLD , &m_matWorld);
	m_pD3DDevice->GetTransform(D3DTS_VIEW , &m_matWorldView);
	m_pD3DDevice->GetTransform(D3DTS_PROJECTION , &m_matProj);
	m_matWorldViewProj = m_matWorldView * m_matProj;

	D3DXMatrixTranspose(&m_matWorldView, &m_matWorldView);
	D3DXMatrixTranspose(&m_matWorldViewProj, &m_matWorldViewProj);

	D3DXVECTOR4 vCamera(eyePosition.x, eyePosition.y, eyePosition.z, 1.0f);

	short a = LOWORD(Color);
	short b = HIWORD(Color);

	byte _r = LOBYTE(a);
	byte _g = LOBYTE(a);
	byte _b = LOBYTE(b);
	byte _a = LOBYTE(b);
	/* (height /  end / --   / -- )*/
//	D3DXVECTOR4 fFog      = D3DXVECTOR4(200.0f, 1500.0f, 0.0f, 0.0f);
	/* light gray */
//	D3DXVECTOR4 colFog    =  D3DXVECTOR4(_r, _g, _b, _a);
							   /* (height /  end / --   / -- )*/
    D3DXVECTOR4 fFog      = D3DXVECTOR4(20.0f, 150.0f, 0.0f, 0.0f);

									/* light gray */
    D3DXVECTOR4 colFog    = D3DXVECTOR4(150.0f/255.0f, 150.0f/255.0f, 150.0f/255.0f, 0.0f/255.0f);
	// Set vertex shader constants
	m_pD3DDevice->SetVertexShaderConstantF(0, (float*)&m_matWorldViewProj,	4);
	m_pD3DDevice->SetVertexShaderConstantF(4, (float*)&m_matWorldView,		4);
	m_pD3DDevice->SetVertexShaderConstantF(8, (float*)&m_matWorld,			4);
	m_pD3DDevice->SetVertexShaderConstantF(12, (float*)&fFog,				1);
	m_pD3DDevice->SetVertexShaderConstantF(13, (float*)&vCamera,			1);

	// Set pixel shader constants
	m_pD3DDevice->SetPixelShaderConstantF(5, (float*)&colFog,				1);

	return S_OK;
}

void Shader_Fog::Rander_Layer(D3DXVECTOR3 eyePosition, int Color)
{
	SetShaderConstants(eyePosition, Color);
	if(NULL != m_pVertexShader)
		m_pD3DDevice->SetVertexShader(m_pVertexShader);
	if(NULL != m_pPixelShader)
		m_pD3DDevice->SetPixelShader(m_pPixelShader);

}

void Shader_Fog::Rander_Begin_Exp2(float exp2, int Color)
{
	FogEffect->SetTechnique( FogTechHandle );

	FogEffect->SetFloat("exp2", exp2);
	FogEffect->SetInt("color", Color);

	UINT numPasses = 0;
	FogEffect->Begin(&numPasses, 0);

	FogEffect->BeginPass(0);
}

void Shader_Fog::Rander_Begin_LINEAR(float start, float end, int Color)
{
	FogEffect->SetTechnique( FogTechHandle );

	FogEffect->SetFloat("start", start);
	FogEffect->SetFloat("end", end);
	FogEffect->SetInt("color", Color);

	UINT numPasses = 0;
	FogEffect->Begin(&numPasses, 0);

	FogEffect->BeginPass(1);
}

void Shader_Fog::Rander_End()
{
	FogEffect->End();
}
