#include "stdafx.h"
#include "CG_Ocean.h"


CG_Ocean::CG_Ocean()
{
	m_pD3DDevice			= NULL;
	m_pVertexBuffer			= NULL;
	m_Texture_OceanBump		= NULL;
	m_Texture_Sky			= NULL;
	m_pVertexDeclaration	= NULL;

	gridsize_x = 128;
	gridsize_y = 128;
}

CG_Ocean::~CG_Ocean()
{
	cgD3D9SetDevice(NULL);
	cgDestroyProgram(g_CGprogram_vertex);
	cgDestroyProgram(g_CGprogram_pixel);
	cgDestroyContext(g_CGcontext);

	if( m_Texture_OceanBump != NULL ) 
		m_Texture_OceanBump->Release(); 

	if( m_pVertexBuffer != NULL )
		m_pVertexBuffer->Release();

//	if( m_pD3DDevice != NULL )
//		m_pD3DDevice->Release();
}

void CG_Ocean::Init(LPDIRECT3DDEVICE9 pDevice, D3DXVECTOR3 position, float Width, float Height,
					const LPWSTR WaterTextureName, LPDIRECT3DTEXTURE9 *SkyTexture)
{
	CLitVertex g_quadVertices[4];

	g_quadVertices[0] = CLitVertex( -(Width / 2), 0.0f,  (Height / 2),  0xffffff00,  0.0f, 0.0f);
	g_quadVertices[1] = CLitVertex(  (Width / 2), 0.0f,  (Height / 2),  0xff00ff00,  1.0f, 0.0f);
	g_quadVertices[2] = CLitVertex( -(Width / 2), 0.0f, -(Height / 2),  0xffff0000,  0.0f, 1.0f);
	g_quadVertices[3] = CLitVertex(  (Width / 2), 0.0f, -(Height / 2),  0xff0000ff,  1.0f, 1.0f);

	//
	// Create a texture to test out our pixel shader...
	//
	m_pD3DDevice = pDevice;
	m_Position = position;
	m_Position.x += (Width/2);
	m_Position.z += (Height/2);
	
	m_pD3DDevice->CreateVertexBuffer( 4*sizeof(CLitVertex), D3DUSAGE_WRITEONLY, 
		LITVERTEX_FVF, D3DPOOL_DEFAULT, 
		&m_pVertexBuffer, NULL );
	void *pVertices = NULL;

	m_pVertexBuffer->Lock( 0, sizeof(g_quadVertices), (void**)&pVertices, 0 );
	memcpy( pVertices, g_quadVertices, sizeof(g_quadVertices) );
	m_pVertexBuffer->Unlock();

/*
	CLitVertex vertices[128 * 128];

	for(int v=0; v < gridsize_y; v++)
	{
		for(int u=0; u < gridsize_x; u++)
		{
			vertices[v*gridsize_x + u].x =	0.0f;
			vertices[v*gridsize_x + u].y =	1.0f;
			vertices[v*gridsize_x + u].z =	0.0f;
			vertices[v*gridsize_x + u].tu = (float)u/(gridsize_x-1);
			vertices[v*gridsize_x + u].tv = (float)v/(gridsize_y-1);
			vertices[v*gridsize_x + u].Diffuse = 0xffffff00;
		}
	}	

	// create the vertexbuffer used in the softwaremode (it can be empty as it'll be memcpy-ed to)
	if(	FAILED( m_pD3DDevice->CreateVertexBuffer( gridsize_x*gridsize_y*sizeof(CLitVertex),
		D3DUSAGE_WRITEONLY, LITVERTEX_FVF,
		D3DPOOL_DEFAULT, &m_pVertexBuffer, NULL ) ) )
	{
		return ;
	}		
	void *pVertices = NULL;

	m_pVertexBuffer->Lock( 0, sizeof(vertices), (void**)&pVertices, 0 );
	memcpy( pVertices, vertices, sizeof(vertices) );
	m_pVertexBuffer->Unlock();

	// create/fill the indexbuffer

	if(	FAILED( m_pD3DDevice->CreateIndexBuffer(	sizeof(unsigned int) * 6 * (gridsize_x-1)*(gridsize_y-1),
		D3DUSAGE_WRITEONLY,			
		D3DFMT_INDEX32,	D3DPOOL_DEFAULT,&surf_indicies,NULL)))
	{
		return ;
	}

	unsigned int *indexbuffer;
	if( FAILED( surf_indicies->Lock(0,0,(void**)&indexbuffer,0 ) ) )
		return ;

	int i = 0;
	{
		for(int v=0; v < gridsize_y-1; v++)
		{
			for(int u=0; u < gridsize_x-1; u++)
			{
				// face 1 |/
				indexbuffer[i++]	= v*gridsize_x + u;
				indexbuffer[i++]	= v*gridsize_x + u + 1;
				indexbuffer[i++]	= (v+1)*gridsize_x + u;

				// face 2 /|
				indexbuffer[i++]	= (v+1)*gridsize_x + u;
				indexbuffer[i++]	= v*gridsize_x + u + 1;
				indexbuffer[i++]	= (v+1)*gridsize_x + u + 1;
			}
		}
	}
	surf_indicies->Unlock();
*/
	m_Texture_Sky = SkyTexture;

	D3DXCreateTextureFromFile( m_pD3DDevice, WaterTextureName, &m_Texture_OceanBump );
	
	//--------------------------------------------------------------------
	// CG ʱȭ 
	// Create the context...
	g_CGcontext = cgCreateContext();

	//
	// Since we're using Cg's expanded interface, we'll need to pass the 
	// Direct3D device to Cg.
	//
	cgD3D9SetDevice( m_pD3DDevice );

	//
	// Determine the best vertex and pixel profiles to use...
	//
	CGprofile vertexProfile = CG_PROFILE_VS_1_1;
	CGprofile pixelProfile  = cgD3D9GetLatestPixelProfile();

	//
	// Grab the optimal options for each profile...
	//

	// Grab the optimal options for each profile...
	const char **vertexOptions[] =
	{
		cgD3D9GetOptimalOptions( vertexProfile ), NULL,
	};
	const char **pixelOptions[] =
	{
		cgD3D9GetOptimalOptions( pixelProfile ), NULL,
	};

	//
	// Create the vertex and pixel shaders...
	//

	g_CGprogram_vertex = cgCreateProgramFromFile( g_CGcontext,
												CG_SOURCE,
												"Effect/CG/OceanEffect.cg",
												vertexProfile,
												"main_vs",
												*vertexOptions );

	g_CGprogram_pixel = cgCreateProgramFromFile( g_CGcontext,
												CG_SOURCE,
												"Effect/CG/OceanEffect.cg",
												pixelProfile,
												"main_fs",
												*pixelOptions );

	//
	// If your program uses explicit binding semantics (like this one), 
	// you can create a vertex declaration using those semantics.
	//
	D3DVERTEXELEMENT9 declaration[] =
	{
		{ 0, 0,  D3DDECLTYPE_FLOAT3,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_POSITION, 0 },
		{ 0, 12, D3DDECLTYPE_D3DCOLOR, D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_COLOR,    0 },
		{ 0, 16, D3DDECLTYPE_FLOAT2,   D3DDECLMETHOD_DEFAULT, D3DDECLUSAGE_TEXCOORD, 0 },
		D3DDECL_END()
	};

	//
	// Ensure the resulting declaration is compatible with the shader...
	//
	cgD3D9ValidateVertexDeclaration( g_CGprogram_vertex, declaration );
	m_pD3DDevice->CreateVertexDeclaration( declaration, &m_pVertexDeclaration );

	//
	// Load the programs using Cg's expanded interface...
	//
	// When the second parameter is set to TRUE, shadowing is enabled. This
	// will allow parameters like our g_CGparam_constColor to be set once and
	// reused with out constantly setting it over and over again.
	//
	cgD3D9LoadProgram( g_CGprogram_vertex, TRUE, 0 );
	cgD3D9LoadProgram( g_CGprogram_pixel, TRUE, 0 );

	//
	// Bind some parameters by name so we can set them later...
	//
	g_CGparam_WorldViewProj = cgGetNamedParameter( g_CGprogram_vertex, "WorldViewProj" );
	g_CGparam_BumpScale		= cgGetNamedParameter( g_CGprogram_vertex, "BumpScale" );
	g_CGparam_bumpSpeed		= cgGetNamedParameter( g_CGprogram_vertex, "bumpSpeed" );
	g_CGparam_textureScale	= cgGetNamedParameter( g_CGprogram_vertex, "textureScale" );
	g_CGparam_time			= cgGetNamedParameter( g_CGprogram_vertex, "time" );
	g_CGparam_waveFreq		= cgGetNamedParameter( g_CGprogram_vertex, "waveFreq" );
	g_CGparam_waveAmp		= cgGetNamedParameter( g_CGprogram_vertex, "waveAmp" );
	g_CGparam_EyePosition	= cgGetNamedParameter( g_CGprogram_vertex, "eyePosition" );


	g_CGparam_deepColor		= cgGetNamedParameter( g_CGprogram_pixel, "deepColor" );
	g_CGparam_shallowColor	= cgGetNamedParameter( g_CGprogram_pixel, "shallowColor" );
	g_CGparam_waterAmount	= cgGetNamedParameter( g_CGprogram_pixel, "waterAmount" );
	g_CGparam_NormalMap		= cgGetNamedParameter( g_CGprogram_pixel, "NormalMap" );

	g_CGparam_SkyMap		= cgGetNamedParameter( g_CGprogram_pixel, "EnvironmentMap" );
	g_CGparam_reflectionColor	= cgGetNamedParameter( g_CGprogram_pixel, "reflectionColor" );
	g_CGparam_reflectionAmount	= cgGetNamedParameter( g_CGprogram_pixel, "reflectionAmount" );
	g_CGparam_reflectionBlur	= cgGetNamedParameter( g_CGprogram_pixel, "reflectionBlur" );
	g_CGparam_fresnelPower	= cgGetNamedParameter( g_CGprogram_pixel, "fresnelPower" );
	g_CGparam_fresnelBias	= cgGetNamedParameter( g_CGprogram_pixel, "fresnelBias" );
	g_CGparam_hdrMultiplier	= cgGetNamedParameter( g_CGprogram_pixel, "hdrMultiplier" );
	
	//
	// Make sure our parameters have the expected size...
	//
	int s = cgD3D9TypeToSize(cgGetParameterType(g_CGparam_WorldViewProj));
	assert(cgD3D9TypeToSize(cgGetParameterType(g_CGparam_WorldViewProj)) == 16 );
}

//-----------------------------------------------------------------------------
// Name: setShaderConstants()
// Desc: 
//-----------------------------------------------------------------------------
void CG_Ocean::setShaderConstants(D3DXVECTOR3 eyePosition)
{
	// ٴǥ ׸  
	D3DXMATRIXA16 world, view, proj;
	m_pD3DDevice->GetTransform(D3DTS_WORLD , &world);
	D3DXMatrixTranslation( &world, m_Position.x, m_Position.y, m_Position.z);

	m_pD3DDevice->GetTransform(D3DTS_VIEW , &view);
	m_pD3DDevice->GetTransform(D3DTS_PROJECTION , &proj);

    D3DXMATRIXA16 worldViewProj = world * view * proj;
    D3DXMatrixTranspose( &worldViewProj, &worldViewProj );

	//
	// Set constants for the vertex shader...
	//
	// Load a combined world-view-projection matrix
	cgD3D9SetUniformMatrix( g_CGparam_WorldViewProj, &worldViewProj );

	//
	// Set constants for the pixel shader...
	//

	cgD3D9SetSamplerState(g_CGparam_NormalMap, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP);
	cgD3D9SetSamplerState(g_CGparam_NormalMap, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP);
	cgD3D9SetSamplerState(g_CGparam_NormalMap, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	cgD3D9SetSamplerState(g_CGparam_NormalMap, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
	cgD3D9SetSamplerState(g_CGparam_NormalMap, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
	cgD3D9SetTextureWrapMode(g_CGparam_NormalMap, 0);

	cgD3D9SetTexture( g_CGparam_NormalMap, m_Texture_OceanBump );


	cgD3D9SetSamplerState(g_CGparam_SkyMap, D3DSAMP_ADDRESSU, D3DTADDRESS_WRAP);
	cgD3D9SetSamplerState(g_CGparam_SkyMap, D3DSAMP_ADDRESSV, D3DTADDRESS_WRAP);
	cgD3D9SetSamplerState(g_CGparam_SkyMap, D3DSAMP_ADDRESSW, D3DTADDRESS_WRAP);
	cgD3D9SetSamplerState(g_CGparam_SkyMap, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
	cgD3D9SetSamplerState(g_CGparam_SkyMap, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
	cgD3D9SetSamplerState(g_CGparam_SkyMap, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);
	cgD3D9SetTextureWrapMode(g_CGparam_SkyMap, 1);

	cgD3D9SetTexture( g_CGparam_SkyMap, *m_Texture_Sky );



	D3DXVECTOR2 trxtureScale(25,26);
	cgD3D9SetUniform( g_CGparam_textureScale, &trxtureScale );

	//	static double dStartTime = timeGetTime();
	//	float fElpasedAppTime = (float)((timeGetTime() - dStartTime) * 0.001);

	static float time_0_x = 0.0f;
	time_0_x >= 25.0f ? time_0_x = -25.0f : time_0_x += 0.005f;
	cgD3D9SetUniform( g_CGparam_time, &time_0_x );

	float waveFreq = 0.028f;
	cgD3D9SetUniform( g_CGparam_waveFreq, &waveFreq );

	float waveAmp = 1.8f;
	cgD3D9SetUniform( g_CGparam_waveAmp, &waveAmp );

	float bumpscale = 0.2f;
	cgD3D9SetUniform( g_CGparam_BumpScale, &bumpscale );

	D3DXVECTOR2 bumpspeed(0.015f,0.005f);
	cgD3D9SetUniform( g_CGparam_bumpSpeed, &bumpspeed );

	//D3DXVECTOR3 eyePostion(0,0,4);
	cgD3D9SetUniform( g_CGparam_EyePosition, &eyePosition );

	D3DXVECTOR4 deepcolor(0.0f, 0.03f, 0.05f, 1.0f);
	cgD3D9SetUniform( g_CGparam_deepColor, &deepcolor );

	D3DXVECTOR4 shallowcolor(0.0f, 0.5f, 0.5f, 1.0f);
	cgD3D9SetUniform( g_CGparam_shallowColor, &shallowcolor );

	D3DXVECTOR4 reflectcolor(0.95f, 1.0f, 1.0f, 1.0f);
	cgD3D9SetUniform( g_CGparam_reflectionColor, &reflectcolor );

	float reflactamount = 1.0f;
	cgD3D9SetUniform( g_CGparam_reflectionAmount, &reflactamount );

	float reflactblur = 0.0f;
	cgD3D9SetUniform( g_CGparam_reflectionBlur, &reflactblur );

	float fresenlpow = 5.0f;
	cgD3D9SetUniform( g_CGparam_fresnelPower, &fresenlpow );

	float fresenlbias = 0.328f;
	cgD3D9SetUniform( g_CGparam_fresnelBias, &fresenlbias );

	float hdrMulti = 0.471f;
	cgD3D9SetUniform( g_CGparam_hdrMultiplier, &hdrMulti );
}

bool CG_Ocean::Rander(D3DXVECTOR3 eyePosition)
{
	// ī޶  .  ִ ġ ٸ.. ׷Ƿ ش.
	D3DXVECTOR3 realPos = eyePosition;
	eyePosition = D3DXVECTOR3(eyePosition.x - m_Position.x, 5.05f, eyePosition.z - m_Position.z);

	// Setup alpha states for rendering water
	m_pD3DDevice->SetTextureStageState( 0, D3DTSS_ALPHAARG1, D3DTA_DIFFUSE );
	m_pD3DDevice->SetTextureStageState( 0, D3DTSS_ALPHAOP, D3DTOP_SELECTARG1 );
	m_pD3DDevice->SetRenderState( D3DRS_SRCBLEND , D3DBLEND_SRCALPHA );
	m_pD3DDevice->SetRenderState( D3DRS_DESTBLEND, D3DBLEND_INVSRCALPHA );
	m_pD3DDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, TRUE );

	// CG  
	setShaderConstants(eyePosition);

	// ؽ ̴ 
	cgD3D9BindProgram( g_CGprogram_vertex );
	// ȼ ̴ 
	cgD3D9BindProgram( g_CGprogram_pixel );

	//     ؽĸ ׸
	m_pD3DDevice->SetFVF(NULL);
	m_pD3DDevice->SetVertexDeclaration( m_pVertexDeclaration );

	m_pD3DDevice->SetStreamSource( 0, m_pVertexBuffer, 0,sizeof(CLitVertex) );
	m_pD3DDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 2 );
/*				
	m_pD3DDevice->SetIndices(surf_indicies);

	m_pD3DDevice->DrawIndexedPrimitive(	D3DPT_TRIANGLELIST, 0,	0, gridsize_x*gridsize_y, 0, 2*(gridsize_x-1)*(gridsize_y-1) );		
*/	//  CG ׸ ̴  Ѵ..
	m_pD3DDevice->SetVertexShader( NULL );
	m_pD3DDevice->SetPixelShader( NULL );

	// Reset states
	m_pD3DDevice->SetRenderState( D3DRS_ZWRITEENABLE, TRUE );
	m_pD3DDevice->SetRenderState( D3DRS_ALPHABLENDENABLE, FALSE );
	m_pD3DDevice->SetRenderState( D3DRS_CULLMODE, D3DCULL_CCW );

    // Render alpha blended quad if we are underwater
    if ( realPos.y - 10.0f < m_Position.y )  
    {
		return true;
        
    } // End if render water effect
	return false;
   
	//
	// Render the normal way...
	//
/*
	D3DXMATRIX g_matWorld;
	D3DXMATRIX matTrans, matRot;

	D3DXMatrixTranslation( &matTrans, 0.0f, 0.0f, 4.0f );
	D3DXMatrixRotationYawPitchRoll( &matRot, 
		D3DXToRadian(0.0f), 
		D3DXToRadian(0.0f), 
		0.0f );
	g_matWorld = matRot * matTrans;

	m_pD3DDevice->SetTransform( D3DTS_WORLD, &g_matWorld );

	m_pD3DDevice->SetFVF( CLitVertex::FVF_Flags );
	m_pD3DDevice->SetVertexShader( NULL );
	m_pD3DDevice->SetPixelShader( NULL );

	m_pD3DDevice->SetTexture( 0, m_Texture_OceanBump );

	m_pD3DDevice->SetStreamSource( 0, m_pVertexBuffer, 0,sizeof(CLitVertex) );
	m_pD3DDevice->DrawPrimitive( D3DPT_TRIANGLESTRIP, 0, 2 );
*/
}
